@@ -1,7 +1,7 @@
 package App::CalendarDatesUtils;
 
-our $DATE = '2019-06-26'; # DATE
-our $VERSION = '0.008'; # VERSION
+our $DATE = '2019-06-28'; # DATE
+our $VERSION = '0.009'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -28,6 +28,13 @@ $SPEC{list_calendar_dates} = {
     v => 1.1,
     summary => 'List dates from one or more Calendar::Dates::* modules',
     args => {
+        action => {
+            schema => ['str*', in=>['list-dates', 'list-modules']],
+            default => 'list-dates',
+            cmdline_aliases => {
+                L => {is_flag=>1, summary=>'List all Calendar::Dates modules (eqv to --action=list-modules)', code=>sub { $_[0]{action} = 'list-modules' }},
+            },
+        },
         year => {
             summary => 'Specify year of dates to list',
             description => <<'_',
@@ -38,16 +45,27 @@ instead to list dates from all available years.
 _
             schema => 'int*',
             pos => 0,
+            cmdline_aliases => {Y=>{}},
+            tags => ['category:entry-filtering'],
+        },
+        min_year => {
+            schema => 'int*',
+            tags => ['category:entry-filtering'],
+        },
+        max_year => {
+            schema => 'int*',
             tags => ['category:entry-filtering'],
         },
         month => {
-            schema => ['int*', in=>[1, 12]],
+            schema => ['int*', between=>[1, 12]],
             pos => 1,
+            cmdline_aliases => {M=>{}},
             tags => ['category:entry-filtering'],
         },
         day => {
-            schema => ['int*', in=>[1, 31]],
+            schema => ['int*', between=>[1, 31]],
             pos => 2,
+            cmdline_aliases => {D=>{}},
             tags => ['category:entry-filtering'],
         },
         all_years => {
@@ -55,6 +73,13 @@ _
                 'instead of a single year',
             schema => 'true*',
             tags => ['category:entry-filtering'],
+            description => <<'_',
+
+Note that by default, following common usage pattern, dates with years that are
+too old (< 10 years ago) or that are too far into the future (> 10 years from
+now) are not included, unless you combine this option with --all-entries (-A).
+
+_
         },
         modules => {
             summary => 'Name(s) of Calendar::Dates::* module (without the prefix)',
@@ -80,6 +105,9 @@ _
 By default, low-priority entries (entries tagged `low-priority`) are not
 included. This option will include those entries.
 
+When combined with --all-years, this option will also cause all very early years
+and all years far into the future to be included also.
+
 _
             tags => ['category:entry-filtering'],
         },
@@ -116,17 +144,24 @@ _
         },
     },
     args_rels => {
-        'req_one&' => [
-            ['modules', 'all_modules'],
-        ],
         'choose_one&' => [
+            ['modules', 'all_modules'],
             ['year', 'all_years'],
+            ['year', 'min_year'],
+            ['year', 'max_year'],
+        ],
+        'choose_all&' => [
         ],
     },
 };
 sub list_calendar_dates {
     my %args = @_;
 
+    my $action = $args{action} // 'list-dates';
+    if ($action eq 'list-modules') {
+        return list_calendar_dates_modules();
+    }
+
     my @lt = localtime;
     my $year_today = $lt[5]+1900;
     my $mon_today  = $lt[4]+1;
@@ -141,8 +176,10 @@ sub list_calendar_dates {
     my $modules;
     if ($args{all_modules}) {
         $modules = list_calendar_dates_modules()->[2];
-    } else {
+    } elsif ($args{modules}) {
         $modules = $args{modules};
+    } else {
+        return [400, "Please specify modules or all_modules"];
     }
 
     my $params = {};
@@ -159,10 +196,28 @@ sub list_calendar_dates {
         require $mod_pm;
 
         my $years;
+        my $min = $mod->get_min_year;
+        my $max = $mod->get_max_year;
         if ($args{all_years}) {
-            $years = [ $mod->get_min_year .. $mod->get_max_year ];
-        } else {
+            if ($min < $year_today - 10 && !$args{all_entries}) {
+                warn "Warning: There are dates with year earlier than ".
+                    ($year_today - 10). " that are not included, ".
+                    "use --all-entries to include them\n";
+                $min = $year_today - 10;
+            }
+            if ($max > $year_today + 10 && !$args{all_entries}) {
+                warn "Warning: There are dates with year later than ".
+                    ($year_today + 10). " that are not included, ".
+                    "use --all-entries to include them\n";
+                $max = $year_today + 10;
+            }
+            $years = [$min..$max];
+        } elsif (defined $args{min_year} || defined $args{max_year}) {
+            $years = [($args{min_year} // $year_today) .. ($args{max_year} // $year_today)];
+        } elsif (defined $args{year}) {
             $years = [ $year ];
+        } else {
+            return [400, "Please specify year, or min_year/max_year, or all_years"];
         }
 
         for my $y (@$years) {
@@ -214,7 +269,7 @@ App::CalendarDatesUtils - Utilities related to Calendar::Dates
 
 =head1 VERSION
 
-This document describes version 0.008 of App::CalendarDatesUtils (from Perl distribution App-CalendarDatesUtils), released on 2019-06-26.
+This document describes version 0.009 of App::CalendarDatesUtils (from Perl distribution App-CalendarDatesUtils), released on 2019-06-28.
 
 =head1 DESCRIPTION
 
@@ -245,6 +300,8 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<action> => I<str> (default: "list-dates")
+
 =item * B<all_entries> => I<true>
 
 Return all entries (include low-priority ones).
@@ -252,6 +309,9 @@ Return all entries (include low-priority ones).
 By default, low-priority entries (entries tagged C<low-priority>) are not
 included. This option will include those entries.
 
+When combined with --all-years, this option will also cause all very early years
+and all years far into the future to be included also.
+
 =item * B<all_modules> => I<true>
 
 Use all installed Calendar::Dates::* modules.
@@ -260,6 +320,10 @@ Use all installed Calendar::Dates::* modules.
 
 List dates from all available years instead of a single year.
 
+Note that by default, following common usage pattern, dates with years that are
+too old (< 10 years ago) or that are too far into the future (> 10 years from
+now) are not included, unless you combine this option with --all-entries (-A).
+
 =item * B<day> => I<int>
 
 =item * B<detail> => I<bool>
@@ -270,6 +334,10 @@ Whether to show detailed record for each date.
 
 =item * B<include_tags> => I<array[str]>
 
+=item * B<max_year> => I<int>
+
+=item * B<min_year> => I<int>
+
 =item * B<modules> => I<array[perl::modname]>
 
 Name(s) of Calendar::Dates::* module (without the prefix).
